@@ -53,6 +53,12 @@ groupBy names df
                 (T.pack $ show $ names L.\\ columnNames df)
                 "groupBy"
                 (columnNames df)
+    | nRows df == 0 =
+        Grouped
+            df
+            names
+            VU.empty
+            (VU.fromList [0])
     | otherwise =
         Grouped
             df
@@ -261,7 +267,7 @@ aggregate aggs gdf@(Grouped df groupingColumns valueIndices offsets) =
                 (VU.map (valueIndices VU.!) (VU.init offsets))
                 (select groupingColumns df)
 
-        f (name, Wrap (expr :: Expr a)) d =
+        f (name, UExpr (expr :: Expr a)) d =
             let
                 value = case interpretAggregation @a gdf expr of
                     Left e -> throw e
